import * as _nuxt_schema from '@nuxt/schema';
import { Types } from '@graphql-codegen/plugin-helpers';
import { SchemaASTConfig } from '@graphql-codegen/schema-ast';
import { TypeScriptDocumentsPluginConfig } from '@graphql-codegen/typescript-operations';
import { H3Event } from 'h3';
import { FetchOptions, FetchResponse, FetchError } from 'ofetch';
import { GraphqlServerResponse } from '#graphql-middleware/types';
import { GraphqlMiddlewareResponseUnion } from '#build/nuxt-graphql-middleware';

type GraphqlMiddlewareGraphqlEndpointMethod = (event?: H3Event, operation?: string, operationName?: string) => string | Promise<string> | undefined;
type GraphqlMiddlewareServerFetchOptionsMethod = (event?: H3Event, operation?: string, operationName?: string) => FetchOptions | Promise<FetchOptions>;
type GraphqlMiddlewareOnServerResponseMethod<ServerReponse, T> = (event: H3Event, response: FetchResponse<ServerReponse>, operation?: string, operationName?: string) => T | Promise<T>;
type GraphqlMiddlewareOnServerErrorMethod = (event: H3Event, error: FetchError, operation?: string, operationName?: string) => any | Promise<any>;
type GraphqlMiddlewareDoRequestMethodContext = {
    /**
     * The incoming request event from H3.
     */
    event: H3Event;
    /**
     * The type of operation.
     */
    operation: 'query' | 'mutation';
    /**
     * The name of the operation.
     */
    operationName: string;
    /**
     * The operation document (the raw GraphQL query/mutation as a string).
     */
    operationDocument: string;
    /**
     * Variables for the operation.
     */
    variables: Record<string, any>;
    /**
     * For file uploads (which are done using FormData), this contains the full
     * form data with these keys:
     *
     * - operations: string
     *   A JSON string of an object with "query", "variables" and "operationName" properties.
     *   Example:
     *   operations='{ "query": "mutation ($id: String!, $file: Upload!) { uploadFile(id: $id, file: $file) { id filename } }", "variables": { "file": null, "id": "1" } }'
     * - map: string
     *   A JSON string of an object whose keys are strings starting at '0' and the values being an array with a single string:
     *   Example:
     *   map='{ "0": ["variables.file"] }'
     * - [number]
     *   For every file there is a FormData entry with a name that matches the key in `map`.
     *   Example:
     *   0=[Binary File]
     */
    formData?: FormData;
};
type GraphqlMiddlewareDoRequestMethod<T> = (context: GraphqlMiddlewareDoRequestMethodContext) => Promise<T>;
/**
 * Configuration options during runtime.
 */
type GraphqlMiddlewareServerOptions<Additions extends object = object, CustomResponse = GraphqlServerResponse<GraphqlMiddlewareResponseUnion> & Additions> = {
    /**
     * Custom callback to return the GraphQL endpoint per request.
     *
     * The method is only called if no `doGraphqlRequest` method is implemented.
     *
     * @default undefined
     *
     * @example
     * ```ts
     * function graphqlEndpoint(event, operation, operationName) {
     *   const language = getLanguageFromRequest(event)
     *   return `https://api.example.com/${language}/graphql`
     * }
     * ```
     */
    graphqlEndpoint?: GraphqlMiddlewareGraphqlEndpointMethod;
    /**
     * Provide the options for the ofetch request to the GraphQL server.
     *
     * The method is only called if no `doGraphqlRequest` method is implemented.
     *
     * @default undefined
     *
     * @example
     * ```ts
     * import { getHeader } from 'h3'
     *
     * // Pass the cookie from the client request to the GraphQL request.
     * function serverFetchOptions(event, operation, operationName) {
     *   return {
     *     headers: {
     *       Cookie: getHeader(event, 'cookie')
     *     }
     *   }
     * }
     * ```
     */
    serverFetchOptions?: GraphqlMiddlewareServerFetchOptionsMethod;
    /**
     * Handle the response from the GraphQL server.
     *
     * The method is only called if no `doGraphqlRequest` method is implemented.
     *
     * You can alter the response, add additional properties to the data, get
     * and set headers, etc.
     *
     * ```ts
     * import type { H3Event } from 'h3'
     * import type { FetchResponse } from 'ofetch'
     *
     * function onServerResponse(event: H3Event, graphqlResponse: FetchResponse) {
     *   // Set a static header.
     *   event.node.res.setHeader('x-nuxt-custom-header', 'A custom header value')
     *
     *   // Pass the set-cookie header from the GraphQL response to the client.
     *   const setCookie = graphqlResponse.headers.get('set-cookie')
     *   if (setCookie) {
     *     event.node.res.setHeader('set-cookie', setCookie)
     *   }
     *
     *   // Add additional properties to the response.
     *   graphqlResponse._data.__customProperty = ['My', 'values']
     *
     *   // Return the GraphQL response.
     *   return graphqlResponse._data
     * }
     * ```
     */
    onServerResponse?: GraphqlMiddlewareOnServerResponseMethod<GraphqlServerResponse<GraphqlMiddlewareResponseUnion>, CustomResponse>;
    /**
     * Handle a fetch error from the GraphQL request.
     *
     * The method is only called if no `doGraphqlRequest` method is implemented.
     *
     * Note that errors are only thrown for responses that are not status
     * 200-299. See https://github.com/unjs/ofetch#%EF%B8%8F-handling-errors for
     * more information.
     *
     * ```ts
     * import { createError } from 'h3'
     * import type { H3Event } from 'h3'
     * import type { FetchError } from 'ofetch'
     *
     * function onServerError(
     *   event: H3Event,
     *   error: FetchError,
     *   operation: string,
     *   operationName: string,
     * ) {
     *   // Throw a h3 error.
     *   throw createError({
     *     statusCode: 500,
     *     statusMessage: `Couldn't execute GraphQL ${operation} "${operationName}".`,
     *     data: error.message
     *   })
     * }
     * ```
     */
    onServerError?: GraphqlMiddlewareOnServerErrorMethod;
    /**
     * Provide a custom fetch method for requests to the GraphQL server.
     *
     * This can be used if onServerError, onServerResponse, serverFetchOptions
     * and graphqlEndpoint are not enough to meet your requirements.
     *
     * When this method is implemented, all other methods are not called.
     *
     * The method will be called in the /api/graphql server route and should
     * perform the GraphQL request and return the response.
     *
     * An example use case might be to handle expired tokens.
     *
     * * ```ts
     * async function doGraphqlRequest({
     *   event,
     *   operation,
     *   operationName,
     *   operationDocument,
     *   variables,
     * }) {
     *   const result = await $fetch.raw('https://example.com/graphql', {
     *     method: 'POST'
     *     body: {
     *       query: operationDocument,
     *       variables,
     *       operationName
     *     },
     *     headers: {
     *       'custom-header': 'foobar'
     *     }
     *   })
     *
     *   return result._data
     * }
     * ```
     */
    doGraphqlRequest?: GraphqlMiddlewareDoRequestMethod<CustomResponse>;
};

interface ModuleOptions {
    /**
     * File glob patterns for the auto import feature.
     *
     * If left empty, no documents are auto imported.
     *
     * @default
     * ```json
     * ["**\/.{gql,graphql}", "!node_modules"]
     * ```
     *
     * @example
     * ```ts
     * // Load .graphql files from pages folder and from a node_modules dependency.
     * const autoImportPatterns = [
     *   './pages/**\/*.graphql',
     *   'node_modules/my_library/dist/**\/*.graphql'
     * ]
     * ```
     */
    autoImportPatterns?: string[];
    /**
     * Automatically inline fragments.
     *
     * By default, fragments have to imported using the non-standard "#import"
     * syntax.
     *
     * When enabling this feature, the module will automatically inline fragments by name.
     */
    autoInlineFragments?: boolean;
    /**
     * Additional raw documents to include.
     *
     * Useful if for example you need to generate queries during build time.
     *
     * @default []
     *
     * @example
     * ```ts
     * const documents = [`
     *   query myQuery {
     *     articles {
     *       title
     *       id
     *     }
     *   }`,
     *   ...getGeneratedDocuments()
     * ]
     * ```
     */
    documents?: string[];
    /**
     * Wether the useGraphqlQuery, useGraphqlMutation and useGraphqlState
     * composables should be included.
     *
     * @default ```ts
     * true
     * ```
     */
    includeComposables?: boolean;
    /**
     * Enable support for uploading files via GraphQL.
     *
     * When enabled, an additional `useGraphqlUploadMutation` composable is
     * included, in addition to a new server endpoint that handles multi part
     * file uploads for GraphQL mutations.
     */
    enableFileUploads?: boolean;
    /**
     * Enable detailled debugging messages.
     *
     * @default false
     */
    debug?: boolean;
    /**
     * The URL of the GraphQL server.
     *
     * For the runtime execution you can provide a method that determines the endpoint
     * during runtime. See the app/graphqlMiddleware.serverOptions.ts documentation
     * for more information.
     */
    graphqlEndpoint: string;
    /**
     * Download the GraphQL schema and store it on disk.
     *
     * @default true
     */
    downloadSchema?: boolean;
    /**
     * The prefix for the server route.
     *
     * @default ```ts
     * "/api/graphql_middleware"
     * ```
     */
    serverApiPrefix?: string;
    /**
     * Path to the GraphQL schema file.
     *
     * If `downloadSchema` is `true`, the downloaded schema is written to this specified path.
     * If `downloadSchema` is `false`, this file must be present in order to generate types.
     *
     * @default './schema.graphql'
     */
    schemaPath?: string;
    /**
     * These options are passed to the graphql-codegen method when generating the
     * TypeScript operations types.
     *
     * {@link https://www.the-guild.dev/graphql/codegen/plugins/typescript/typescript-operations}
     * @default
     * ```ts
     * const codegenConfig = {
     *   exportFragmentSpreadSubTypes: true,
     *   preResolveTypes: true,
     *   skipTypeNameForRoot: true,
     *   skipTypename: true,
     *   useTypeImports: true,
     *   onlyOperationTypes: true,
     *   namingConvention: {
     *     enumValues: 'change-case-all#upperCaseFirst',
     *   },
     * }
     * ```
     */
    codegenConfig?: TypeScriptDocumentsPluginConfig;
    /**
     * Configuration for graphql-codegen when downloading the schema.
     */
    codegenSchemaConfig?: {
        /**
         * Configure how the schema.graphql file should be generated.
         */
        schemaAstConfig?: SchemaASTConfig;
        /**
         * Configure how the schema-ast introspection request should be made.
         *
         * Usually this is where you can provide a custom authentication header:
         *
         * ```typescript
         * const codegenSchemaConfig = {
         *   urlSchemaOptions: {
         *     headers: {
         *       authentication: 'foobar',
         *     }
         *   }
         * }
         * ```
         */
        urlSchemaOptions?: Types.UrlSchemaOptions;
    };
    /**
     * Set to true if you want to output each compiled query and mutation in the
     * .nuxt folder.
     */
    outputDocuments?: boolean;
    /**
     * Enable Nuxt DevTools integration.
     */
    devtools?: boolean;
    /**
     * Client caching configuration.
     */
    clientCache?: {
        enabled?: boolean;
        maxSize?: number;
    };
}
interface ModuleHooks {
}
declare const _default: _nuxt_schema.NuxtModule<ModuleOptions>;

declare module 'nuxt/schema' {
    interface AppConfig {
        graphqlMiddleware: {
            clientCacheEnabled: boolean;
            clientCacheMaxSize: number;
        };
    }
}

export { type GraphqlMiddlewareServerOptions, type ModuleHooks, type ModuleOptions, _default as default };
