import { fileURLToPath } from 'url';
import { dirname, resolve, relative } from 'pathe';
import { defu } from 'defu';
import { useLogger, resolveAlias, resolveFiles, defineNuxtModule, createResolver, addImports, addTemplate, addServerHandler, addPlugin, updateTemplates } from '@nuxt/kit';
import inquirer from 'inquirer';
import { onDevToolsInitialized, extendServerRpc } from '@nuxt/devtools-kit';
import { existsSync } from 'fs';
import { GraphqlMiddlewareTemplate } from '../dist/runtime/settings/index.js';
import * as fs from 'node:fs';
import { promises, existsSync as existsSync$1 } from 'node:fs';
import { oldVisit } from '@graphql-codegen/plugin-helpers';
import { validateGraphQlDocuments } from '@graphql-tools/utils';
import { loadSchema } from '@graphql-tools/load';
import { concatAST, parse, print, visit, Kind, Source } from 'graphql';
import { falsy } from '../dist/runtime/helpers/index.js';
import { generate as generate$1, executeCodegen } from '@graphql-codegen/cli';
import * as PluginTypescript from '@graphql-codegen/typescript';
import * as PluginTypescriptOperations from '@graphql-codegen/typescript-operations';
import * as PluginSchemaAst from '@graphql-codegen/schema-ast';
import { pascalCase } from 'change-case-all';
import colors from 'picocolors';

const name = "nuxt-graphql-middleware";
const version = "4.1.1";

const DEVTOOLS_UI_ROUTE = "/__nuxt-graphql-middleware";
const DEVTOOLS_UI_LOCAL_PORT = 3300;
function setupDevToolsUI(nuxt, clientPath) {
  const isProductionBuild = existsSync(clientPath);
  if (isProductionBuild) {
    nuxt.hook("vite:serverCreated", async (server) => {
      const sirv = await import('sirv').then((r) => r.default || r);
      server.middlewares.use(
        DEVTOOLS_UI_ROUTE,
        sirv(clientPath, { dev: true, single: true })
      );
    });
  } else {
    nuxt.hook("vite:extendConfig", (config) => {
      config.server = config.server || {};
      config.server.proxy = config.server.proxy || {};
      config.server.proxy[DEVTOOLS_UI_ROUTE] = {
        target: "http://localhost:" + DEVTOOLS_UI_LOCAL_PORT + DEVTOOLS_UI_ROUTE,
        changeOrigin: true,
        followRedirects: true,
        rewrite: (path) => path.replace(DEVTOOLS_UI_ROUTE, "")
      };
    });
  }
  nuxt.hook("devtools:customTabs", (tabs) => {
    tabs.push({
      // unique identifier
      name: "nuxt-graphql-middleware",
      // title to display in the tab
      title: "GraphQL Middleware",
      // any icon from Iconify, or a URL to an image
      icon: "akar-icons:graphql-fill",
      // iframe view
      view: {
        type: "iframe",
        src: DEVTOOLS_UI_ROUTE
      }
    });
  });
}

const importSyntaxRE = /^#import (?:'([^']*)'|"([^"]*)")/;
const matchImport = (value = "") => {
  if (!value) {
    return void 0;
  }
  if (value.indexOf("#import") !== 0) {
    return void 0;
  }
  const matched = value.match(importSyntaxRE);
  if (matched === null) {
    return void 0;
  }
  const importIdentifierMatch = value.match(importSyntaxRE) || [];
  const importIdentifier = importIdentifierMatch[1] ?? importIdentifierMatch[2];
  if (importIdentifier === void 0) {
    return void 0;
  }
  return { importIdentifier };
};

const lineEndingRE = /\r\n|\n/;
function* linesWithInlinedImportsOf(fileContents, inlineImportsOptions, visited) {
  const {
    resolveOptions = {},
    resolveImport,
    fs: nodeFs = fs,
    throwIfImportNotFound
  } = inlineImportsOptions;
  const { basedir } = resolveOptions;
  if (typeof basedir !== "string") {
    throw new TypeError(
      "inlineImports requires options.resolverOptions.basedir be set"
    );
  }
  if (!resolveImport) {
    throw new TypeError("inlineImports requires options.resolveImport be set");
  }
  let lineNumber = 0;
  for (const line of fileContents.split(lineEndingRE)) {
    ++lineNumber;
    const matched = matchImport(line);
    if (matched) {
      const importIdentifier = matched.importIdentifier;
      let filename;
      try {
        filename = resolveImport(importIdentifier, resolveOptions);
      } catch (err) {
        if (throwIfImportNotFound === false) {
          continue;
        }
        throw err;
      }
      if (visited.has(filename)) {
        continue;
      } else {
        visited.add(filename);
      }
      const fragmentSource = nodeFs.readFileSync(filename, "utf8");
      const line2 = inlineImportsWithLineToImports(
        fragmentSource,
        {
          resolveImport,
          resolveOptions: {
            basedir: dirname(filename)
          }
        },
        visited
      );
      yield { line: line2.inlineImports, match: true, lineNumber, filename };
    } else {
      yield { line, match: false, lineNumber };
    }
  }
}
function inlineImportsWithLineToImports(fileContents, options, visited = /* @__PURE__ */ new Set()) {
  const inlineImportsResult = [];
  const lineToImports = /* @__PURE__ */ new Map();
  for (const { line, match, lineNumber, filename } of linesWithInlinedImportsOf(
    fileContents,
    options,
    visited
  )) {
    inlineImportsResult.push(line);
    if (match) {
      lineToImports.set(lineNumber, { filename, line });
    }
  }
  return {
    inlineImports: inlineImportsResult.join("\n"),
    lineToImports
  };
}

function getCodeResult(operations, typeName, serverApiPrefix) {
  const imports = [];
  const resultTypes = [];
  let code = "";
  let nitroCode = "";
  const names = Object.keys(operations);
  if (names.length) {
    const lines = [];
    const nitroLines = [];
    names.forEach((name) => {
      const nameResult = pascalCase(name + typeName);
      resultTypes.push(nameResult);
      imports.push(nameResult);
      const nameVariables = pascalCase(name + typeName + "Variables");
      const { hasVariables, variablesOptional } = operations[name];
      if (hasVariables) {
        imports.push(nameVariables);
      }
      const variablesType = hasVariables ? nameVariables : "null";
      lines.push(
        `    ${name}: [${variablesType}, ${variablesOptional ? "true" : "false"}, ${nameResult}]`
      );
      nitroLines.push(
        `    '${serverApiPrefix}/${typeName.toLowerCase()}/${name}': {
      'default': GraphqlResponse<${nameResult}>
    }`
      );
    });
    code += `  export type GraphqlMiddleware${typeName} = {
${lines.join(",\n")}
  }
`;
    nitroCode += `${nitroLines.join("\n")}`;
  }
  return { code, imports, nitroCode, resultTypes };
}
const plugin$1 = (_schema, documents, config) => {
  const allAst = concatAST(documents.map((v) => v.document).filter(falsy));
  const operations = {
    query: {},
    mutation: {}
  };
  oldVisit(allAst, {
    enter: {
      OperationDefinition: (node) => {
        if ("name" in node && node.name?.value && "operation" in node && (node.operation === "query" || node.operation === "mutation")) {
          operations[node.operation][node.name.value] = {
            hasVariables: !!node.variableDefinitions?.length,
            variablesOptional: !!node.variableDefinitions?.every((v) => {
              return v.defaultValue;
            })
          };
        }
      }
    }
  });
  let code = "";
  let nitroCode = "";
  const imports = [];
  const resultTypes = [];
  const resultQuery = getCodeResult(
    operations.query,
    "Query",
    config.serverApiPrefix
  );
  code += resultQuery.code;
  nitroCode += resultQuery.nitroCode;
  imports.push(...resultQuery.imports);
  resultTypes.push(...resultQuery.resultTypes);
  const resultMutation = getCodeResult(
    operations.mutation,
    "Mutation",
    config.serverApiPrefix
  );
  code += "\n" + resultMutation.code;
  nitroCode += "\n" + resultMutation.nitroCode;
  imports.push(...resultMutation.imports);
  resultTypes.push(...resultMutation.resultTypes);
  return `
import type { GraphqlResponse } from '#graphql-middleware-server-options-build'
import type {
  ${imports.join(",\n  ")}
} from './graphql-operations'


declare module '#build/nuxt-graphql-middleware' {
  export type GraphqlMiddlewareResponseUnion = ${resultTypes.join(" | ")}
${code}
}

declare module 'nitropack' {
  interface InternalApi {
${nitroCode}
  }
}
`;
};

const PluginNuxtGraphqlMiddleware = {
  __proto__: null,
  plugin: plugin$1
};

const plugin = (_schema, documents, _config) => {
  const allAst = concatAST(documents.map((v) => v.document).filter(falsy));
  const operations = {
    query: {},
    mutation: {}
  };
  oldVisit(allAst, {
    enter: {
      OperationDefinition: (node) => {
        if (node.name?.value && node.loc?.source && (node.operation === "query" || node.operation === "mutation")) {
          operations[node.operation][node.name.value] = node.loc.source.body;
        }
      }
    }
  });
  return `const documents = ${JSON.stringify(operations, null, 2)};
export { documents };`;
};

const PluginNuxtGraphqlMiddlewareDocuments = {
  __proto__: null,
  plugin: plugin
};

function pluginLoader(name) {
  switch (name) {
    case "@graphql-codegen/typescript":
      return Promise.resolve(PluginTypescript);
    case "@graphql-codegen/typescript-operations":
      return Promise.resolve(PluginTypescriptOperations);
    case "@graphql-codegen/typescript-nuxt-graphql-middleware":
      return Promise.resolve(PluginNuxtGraphqlMiddleware);
    case "@graphql-codegen/typescript-nuxt-graphql-middleware-documents":
      return Promise.resolve(PluginNuxtGraphqlMiddlewareDocuments);
    case "@graphql-codegen/schema-ast":
      return Promise.resolve(PluginSchemaAst);
  }
  throw new Error(`graphql-codegen plugin not found: ${name}`);
}
function generateSchema(moduleOptions, dest, writeToDisk) {
  const pluginConfig = moduleOptions.codegenSchemaConfig?.urlSchemaOptions;
  const schemaAstConfig = moduleOptions.codegenSchemaConfig?.schemaAstConfig || {
    sort: true
  };
  const config = {
    schema: moduleOptions.graphqlEndpoint,
    pluginLoader,
    silent: true,
    errorsOnly: true,
    config: pluginConfig,
    generates: {
      [dest]: {
        plugins: ["schema-ast"],
        config: schemaAstConfig
      }
    }
  };
  return generate$1(config, writeToDisk).then((v) => v[0]);
}
function generateTemplates(documents, schemaPath, options) {
  return executeCodegen({
    schema: schemaPath,
    pluginLoader,
    silent: true,
    errorsOnly: true,
    documents,
    generates: {
      [GraphqlMiddlewareTemplate.OperationTypes]: {
        plugins: ["typescript", "typescript-operations"],
        config: options.codegenConfig
      },
      [GraphqlMiddlewareTemplate.ComposableContext]: {
        plugins: ["typescript-nuxt-graphql-middleware"],
        config: {
          serverApiPrefix: options.serverApiPrefix
        }
      },
      [GraphqlMiddlewareTemplate.Documents]: {
        plugins: ["typescript-nuxt-graphql-middleware-documents"]
      }
    }
  });
}

function getMaxLengths(documents) {
  let longestOperation = 0;
  let longestName = 0;
  let longestPath = 0;
  for (const { operation, name, relativePath } of documents) {
    if (operation && operation.length > longestOperation) {
      longestOperation = operation.length;
    }
    if (name && name.length > longestName) {
      longestName = name.length;
    }
    if (relativePath && relativePath.length > longestPath) {
      longestPath = relativePath.length;
    }
  }
  return { longestOperation, longestName, longestPath };
}
function logDocuments(logger, documents, logEverything) {
  const { longestOperation, longestName, longestPath } = getMaxLengths(documents);
  logger.log(colors.green("GraphQL Document Validation"));
  for (const { operation, name, relativePath, isValid, errors } of documents) {
    if (logEverything || !isValid) {
      let log = "";
      log += (operation || "").padEnd(longestOperation + 2);
      log += colors.cyan((name || "").padEnd(longestName + 2));
      log += colors.dim((relativePath || "").padEnd(longestPath + 2));
      log += isValid ? colors.green("\u2713") : colors.red("x");
      if (!isValid && errors) {
        log += "\n" + errors.map((error) => colors.red(error)).join("\n");
      }
      logger.log(log);
    }
  }
  process.stdout.write("\n");
  logger.restoreStd();
  if (documents.some((v) => !v.isValid)) {
    logger.error("GraphQL document validation failed with errors.");
  } else {
    logger.success("GraphQL document validation completed successfully.");
  }
}

const logger = useLogger("nuxt-graphql-middleware");
const defaultOptions = {
  codegenConfig: {
    exportFragmentSpreadSubTypes: true,
    preResolveTypes: true,
    skipTypeNameForRoot: true,
    skipTypename: true,
    useTypeImports: true,
    onlyOperationTypes: true,
    namingConvention: {
      enumValues: "change-case-all#upperCaseFirst"
    }
  },
  downloadSchema: true,
  schemaPath: "./schema.graphql",
  serverApiPrefix: "/api/graphql_middleware",
  graphqlEndpoint: "",
  debug: false,
  includeComposables: true,
  documents: [],
  devtools: true
};
function inlineFragments(source, resolver) {
  return inlineImportsWithLineToImports(source, {
    resolveImport(identifier) {
      return resolver(identifier);
    },
    resolveOptions: {
      basedir: "./"
    }
  }).inlineImports;
}
function validateOptions(options) {
  if (!options.graphqlEndpoint) {
    throw new Error("Missing graphqlEndpoint.");
  }
}
async function getSchemaPath(options, resolver, writeToDisk = false) {
  const dest = resolver(options.schemaPath);
  if (!options.downloadSchema) {
    const fileExists2 = await promises.access(dest).then(() => true).catch(() => false);
    if (!fileExists2) {
      logger.error(
        '"downloadSchema" is set to false but no schema exists at ' + dest
      );
      throw new Error("Missing GraphQL schema.");
    }
    return dest;
  }
  if (!options.graphqlEndpoint) {
    throw new Error("Missing graphqlEndpoint config.");
  }
  await generateSchema(options, dest, writeToDisk);
  return dest;
}
async function autoImportDocuments(patterns = [], srcResolver) {
  if (!patterns.length) {
    return Promise.resolve([]);
  }
  const files = (await resolveFiles(srcResolver(), patterns, {
    followSymbolicLinks: false
  })).filter((path) => {
    return !path.includes("schema.gql") && !path.includes("schema.graphql");
  });
  return Promise.all(
    files.map((filename) => {
      return promises.readFile(filename).then((v) => {
        const content = v.toString().trim();
        return {
          content,
          filename
        };
      });
    })
  );
}
function inlineNestedFragments(document, fragmentMap) {
  const parsed = parse(document);
  const fragmentsToInline = /* @__PURE__ */ new Set();
  visit(parsed, {
    FragmentSpread(node) {
      fragmentsToInline.add(node.name.value);
    }
  });
  fragmentsToInline.forEach((fragmentName) => {
    const fragment = fragmentMap[fragmentName];
    if (fragment) {
      document += "\n" + fragment;
      const nestedFragmentNames = /* @__PURE__ */ new Set();
      visit(parse(fragment), {
        FragmentSpread(node) {
          nestedFragmentNames.add(node.name.value);
        }
      });
      nestedFragmentNames.forEach((nestedFragmentName) => {
        if (!fragmentsToInline.has(nestedFragmentName)) {
          fragmentsToInline.add(nestedFragmentName);
          const nestedFragment = fragmentMap[nestedFragmentName];
          if (nestedFragment) {
            document += "\n" + nestedFragment;
          }
        }
      });
    }
  });
  return document;
}
async function buildDocuments(providedDocuments = [], autoImportPatterns, resolver, autoInlineFragments) {
  const documents = await autoImportDocuments(autoImportPatterns, resolver).then((importedDocuments) => [
    ...importedDocuments,
    ...providedDocuments.map((content) => ({
      content,
      filename: "nuxt.config.ts"
    }))
  ]).then((documents2) => {
    if (autoInlineFragments) {
      return documents2;
    }
    return documents2.map((v) => {
      try {
        return {
          content: inlineFragments(v.content, resolveAlias),
          filename: v.filename
        };
      } catch (e) {
        logger.error(e);
        logger.error(
          "Failed to inline fragments for document: " + v.filename
        );
      }
      return null;
    }).filter(falsy);
  });
  if (!autoInlineFragments) {
    return documents;
  }
  const fragmentMap = {};
  documents.forEach((doc) => {
    const parsed = parse(doc.content);
    visit(parsed, {
      FragmentDefinition(node) {
        fragmentMap[node.name.value] = print(node);
      }
    });
  });
  documents.forEach((doc) => {
    doc.content = inlineNestedFragments(doc.content, fragmentMap);
  });
  return documents;
}
function parseDocument(document, srcDir) {
  let name = document.filename ? document.filename.replace(srcDir, "") : "";
  if (name.charAt(0) === "/") {
    name = name.slice(1);
  }
  const source = new Source(document.content, name);
  return parse(source);
}
function validateDocuments(schema, documents, srcDir) {
  const validated = [];
  for (let i = 0; i < documents.length; i++) {
    const document = { ...documents[i] };
    if (document.filename) {
      document.relativePath = document.filename.replace(srcDir + "/", "");
    }
    try {
      const node = parseDocument(document, srcDir);
      document.content = print(node);
      document.errors = validateGraphQlDocuments(schema, [
        node
      ]);
      const operation = node.definitions.find(
        (v) => v.kind === "OperationDefinition"
      );
      if (operation) {
        document.name = operation.name?.value;
        document.operation = operation.operation;
      } else {
        document.name = document.relativePath;
      }
      document.isValid = document.errors.length === 0;
    } catch (e) {
      document.errors = [e];
      document.isValid = false;
    }
    document.id = [document.operation, document.name, document.filename].filter(Boolean).join("_");
    validated.push(document);
    if (!document.isValid) {
      break;
    }
  }
  return validated;
}
function cleanGraphqlDocument(graphqlContent, operationName) {
  const document = parse(graphqlContent);
  let selectedOperation = null;
  const fragments = {};
  const usedFragments = /* @__PURE__ */ new Set();
  visit(document, {
    OperationDefinition(node) {
      if (node.name?.value === operationName) {
        selectedOperation = node;
      }
    },
    FragmentDefinition(node) {
      fragments[node.name.value] = node;
    }
  });
  if (!selectedOperation) {
    throw new Error(`Operation named "${operationName}" not found`);
  }
  visit(selectedOperation, {
    FragmentSpread(node) {
      usedFragments.add(node.name.value);
    }
  });
  let hasNewFragments = true;
  while (hasNewFragments) {
    hasNewFragments = false;
    for (const fragmentName of usedFragments) {
      visit(fragments[fragmentName], {
        FragmentSpread(node) {
          if (!usedFragments.has(node.name.value)) {
            usedFragments.add(node.name.value);
            hasNewFragments = true;
          }
        }
      });
    }
  }
  return {
    kind: Kind.DOCUMENT,
    definitions: [
      selectedOperation,
      ...Array.from(usedFragments).map(
        (fragmentName) => fragments[fragmentName]
      )
    ]
  };
}
async function generate(options, schemaPath, resolver, srcDir, logEverything = false) {
  const schemaContent = await promises.readFile(schemaPath).then((v) => v.toString());
  const schema = await loadSchema(schemaContent, { loaders: [] });
  const documents = await buildDocuments(
    options.documents,
    options.autoImportPatterns,
    resolver,
    !!options.autoInlineFragments
  );
  const validated = validateDocuments(schema, documents, srcDir);
  const extracted = validated.filter(
    (v) => !v.operation
  );
  for (let i = 0; i < validated.length; i++) {
    const v = validated[i];
    if (v.isValid) {
      try {
        const node = parse(v.content);
        oldVisit(node, {
          enter: {
            OperationDefinition: (node2) => {
              if (node2.name?.value && node2.loc?.source && (node2.operation === "query" || node2.operation === "mutation")) {
                const document = { ...v };
                const cleaned = cleanGraphqlDocument(
                  node2.loc.source.body,
                  node2.name.value
                );
                const errors = validateGraphQlDocuments(schema, [cleaned]);
                document.errors = document.errors || [];
                document.errors.push(...errors);
                document.isValid = !document.errors.length;
                document.name = node2.name.value;
                document.operation = node2.operation;
                document.content = print(cleaned);
                document.id = [
                  document.operation,
                  document.name,
                  document.filename
                ].filter(Boolean).join("_");
                extracted.push(document);
              }
            }
          }
        });
      } catch (e) {
        logger.error(e);
        extracted.push(v);
        break;
      }
    } else {
      extracted.push(v);
      break;
    }
  }
  const templates = await generateTemplates(
    extracted.filter((v) => v.isValid).map((v) => v.content),
    schemaPath,
    options
  );
  const hasErrors = extracted.some((v) => !v.isValid) || validated.some((v) => !v.isValid);
  if (hasErrors || logEverything) {
    logDocuments(logger, extracted, logEverything);
  }
  process.stdout.write("\n");
  logger.restoreStd();
  hasErrors ? logger.error("GraphQL code generation failed with errors.") : logger.success("Finished GraphQL code generation.");
  return {
    templates: templates.sort((a, b) => {
      return a.filename.localeCompare(b.filename);
    }),
    hasErrors,
    documents: extracted.sort((a, b) => {
      const nameA = a.name || "";
      const nameB = b.name || "";
      return nameA.localeCompare(nameB);
    })
  };
}
const fileExists = (path, extensions = ["js", "ts"]) => {
  if (!path) {
    return null;
  } else if (existsSync$1(path)) {
    return path;
  }
  const extension = extensions.find(
    (extension2) => existsSync$1(`${path}.${extension2}`)
  );
  return extension ? `${path}.${extension}` : null;
};
async function outputDocuments(outputPath, documents) {
  await promises.mkdir(outputPath, { recursive: true });
  documents.forEach((v) => {
    if (v.operation && v.name) {
      const fileName = [v.operation, v.name, "graphql"].join(".");
      const filePath = resolve(outputPath, fileName);
      promises.writeFile(filePath, v.content);
    }
  });
}

const RPC_NAMESPACE = "nuxt-graphql-middleware";
const module = defineNuxtModule({
  meta: {
    name,
    configKey: "graphqlMiddleware",
    version,
    compatibility: {
      nuxt: ">=3.1.0"
    }
  },
  defaults: defaultOptions,
  async setup(passedOptions, nuxt) {
    const options = defu({}, passedOptions, defaultOptions);
    if (!passedOptions.autoImportPatterns) {
      options.autoImportPatterns = ["**/*.{gql,graphql}", "!node_modules"];
    }
    validateOptions(options);
    const moduleResolver = createResolver(import.meta.url);
    const srcDir = nuxt.options.srcDir;
    const srcResolver = createResolver(srcDir).resolve;
    const schemaPath = await getSchemaPath(
      options,
      srcResolver,
      options.downloadSchema
    );
    const runtimeDir = fileURLToPath(new URL("./runtime", import.meta.url));
    nuxt.options.build.transpile.push(runtimeDir);
    const ctx = {
      templates: [],
      documents: []
    };
    let rpc = null;
    if (options.devtools) {
      const clientPath = moduleResolver.resolve("./client");
      setupDevToolsUI(nuxt, clientPath);
      const setupRpc = () => {
        rpc = extendServerRpc(RPC_NAMESPACE, {
          // register server RPC functions
          getModuleOptions() {
            return options;
          },
          getDocuments() {
            return ctx.documents;
          }
        });
      };
      try {
        setupRpc();
      } catch (_e) {
        onDevToolsInitialized(() => {
          setupRpc();
        });
      }
    }
    let prompt = null;
    const generateHandler = async (isFirst = false) => {
      if (prompt && prompt.ui) {
        prompt.ui.close();
        prompt = null;
      }
      try {
        const { templates, hasErrors, documents } = await generate(
          options,
          schemaPath,
          srcResolver,
          srcDir,
          isFirst
        );
        ctx.templates = templates;
        ctx.documents = documents;
        rpc?.broadcast.documentsUpdated(documents);
        if (options.outputDocuments) {
          const destFolder = resolve(
            nuxt.options.buildDir,
            "nuxt-graphql-middleware/documents"
          );
          outputDocuments(destFolder, documents);
          if (isFirst) {
            logger.info("Documents generated at " + destFolder);
          }
        }
        if (hasErrors) {
          throw new Error("Documents has errors.");
        }
      } catch (e) {
        console.log(e);
        logger.error("Failed to generate GraphQL files.");
        if (isFirst) {
          process.exit(1);
        }
        if (!options.downloadSchema) {
          return;
        }
        if (!nuxt.options.dev) {
          return;
        }
        process.stdout.write("\n");
        logger.restoreStd();
        prompt = inquirer.prompt({
          type: "confirm",
          name: "accept",
          message: "Do you want to reload the GraphQL schema?"
        });
        prompt.then(async ({ accept }) => {
          if (accept) {
            await getSchemaPath(options, srcResolver, true);
            await generateHandler();
          }
        });
      }
    };
    await generateHandler(true);
    nuxt.options.runtimeConfig.public["nuxt-graphql-middleware"] = {
      serverApiPrefix: options.serverApiPrefix
    };
    nuxt.options.appConfig.graphqlMiddleware = {
      clientCacheEnabled: !!options.clientCache?.enabled,
      clientCacheMaxSize: options.clientCache?.maxSize || 100
    };
    nuxt.options.runtimeConfig.graphqlMiddleware = {
      graphqlEndpoint: options.graphqlEndpoint || ""
    };
    if (options.includeComposables) {
      addImports({
        from: moduleResolver.resolve("./runtime/composables/useGraphqlQuery"),
        name: "useGraphqlQuery"
      });
      addImports({
        from: moduleResolver.resolve(
          "./runtime/composables/useGraphqlMutation"
        ),
        name: "useGraphqlMutation"
      });
      addImports({
        from: moduleResolver.resolve("./runtime/composables/useGraphqlState"),
        name: "useGraphqlState"
      });
      addImports({
        from: moduleResolver.resolve(
          "./runtime/composables/useAsyncGraphqlQuery"
        ),
        name: "useAsyncGraphqlQuery"
      });
      nuxt.options.alias["#graphql-composable"] = moduleResolver.resolve(
        "runtime/composables/server"
      );
      if (options.enableFileUploads) {
        addImports({
          from: moduleResolver.resolve(
            "./runtime/composables/useGraphqlUploadMutation"
          ),
          name: "useGraphqlUploadMutation"
        });
      }
    }
    Object.values(GraphqlMiddlewareTemplate).forEach((filename) => {
      const result = addTemplate({
        write: true,
        filename,
        options: {
          nuxtGraphqlMiddleware: true
        },
        getContents: () => {
          return ctx.templates.find((v) => v.filename === filename)?.content || "";
        }
      });
      if (result.dst.includes(GraphqlMiddlewareTemplate.Documents)) {
        nuxt.options.alias["#graphql-documents"] = result.dst;
      } else if (result.dst.includes(GraphqlMiddlewareTemplate.OperationTypes)) {
        nuxt.options.alias["#graphql-operations"] = result.dst;
      }
    });
    addTemplate({
      write: true,
      filename: "graphql-documents.d.ts",
      getContents: () => {
        return `
import type {
  GraphqlMiddlewareQuery,
  GraphqlMiddlewareMutation,
} from '#build/nuxt-graphql-middleware'

declare module '#graphql-documents' {
  type Documents = {
    query: GraphqlMiddlewareQuery
    mutation: GraphqlMiddlewareMutation
  }
  const documents: Documents
  export { documents, Documents }
}
`;
      }
    });
    const extensions = ["js", "mjs", "ts"];
    const resolvedPath = "~/app/graphqlMiddleware.serverOptions".replace(/^(~~|@@)/, nuxt.options.rootDir).replace(/^(~|@)/, nuxt.options.srcDir);
    const template = (() => {
      const resolvedFilename = `graphqlMiddleware.serverOptions.ts`;
      const maybeUserFile = fileExists(resolvedPath, extensions);
      const moduleTypesPath = relative(
        nuxt.options.buildDir,
        moduleResolver.resolve("./types")
      );
      const serverOptionsLine = maybeUserFile ? `import serverOptions from './../app/graphqlMiddleware.serverOptions'` : `const serverOptions: GraphqlMiddlewareServerOptions = {}`;
      return addTemplate({
        filename: resolvedFilename,
        write: true,
        getContents: () => `
import type { GraphqlMiddlewareServerOptions } from '${moduleTypesPath}'
${serverOptionsLine}
import type { GraphqlServerResponse } from '#graphql-middleware/types'
import type { GraphqlMiddlewareResponseUnion } from '#build/nuxt-graphql-middleware'

type GraphqlResponseAdditions =
  typeof serverOptions extends GraphqlMiddlewareServerOptions<infer R> ? R : {}

export type GraphqlResponse<T> = GraphqlServerResponse<T> & GraphqlResponseAdditions

export type GraphqlResponseTyped = GraphqlResponse<GraphqlMiddlewareResponseUnion>

export { serverOptions }
`
      });
    })();
    nuxt.options.nitro.externals = nuxt.options.nitro.externals || {};
    nuxt.options.nitro.externals.inline = nuxt.options.nitro.externals.inline || [];
    nuxt.options.nitro.externals.inline.push(template.dst);
    nuxt.options.alias["#graphql-middleware-server-options-build"] = template.dst;
    nuxt.options.alias["#graphql-middleware/types"] = moduleResolver.resolve("./runtime/types");
    addServerHandler({
      handler: moduleResolver.resolve("./runtime/serverHandler/index"),
      route: options.serverApiPrefix + "/:operation/:name"
    });
    if (options.enableFileUploads) {
      addServerHandler({
        handler: moduleResolver.resolve("./runtime/serverHandler/upload"),
        route: options.serverApiPrefix + "/upload/:name"
      });
    }
    addPlugin(moduleResolver.resolve("./runtime/plugins/provideState"), {
      append: false
    });
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.externals = defu(
        typeof nitroConfig.externals === "object" ? nitroConfig.externals : {},
        {
          inline: [moduleResolver.resolve("./runtime")]
        }
      );
    });
    if (nuxt.options.dev || nuxt.options._prepare) {
      addServerHandler({
        handler: moduleResolver.resolve("./runtime/serverHandler/debug"),
        route: options.serverApiPrefix + "/debug"
      });
      nuxt.hook("nitro:build:before", (nitro) => {
        nuxt.hook("builder:watch", async (_event, path) => {
          path = relative(
            nuxt.options.srcDir,
            resolve(nuxt.options.srcDir, path)
          );
          if (!path.match(/\.(gql|graphql)$/)) {
            return;
          }
          if (schemaPath.includes(path)) {
            return;
          }
          await generateHandler();
          await updateTemplates({
            filter: (template2) => {
              return template2.options && template2.options.nuxtGraphqlMiddleware;
            }
          });
          await nitro.hooks.callHook("dev:reload");
        });
      });
    }
  }
});

export { module as default };
