import {
  defineEventHandler,
  readMultipartFormData
} from "h3";
import {
  getEndpoint,
  getFetchOptions,
  validateRequest,
  onServerResponse,
  onServerError,
  throwError
} from "./helpers/index.js";
import { GraphqlMiddlewareOperation } from "./../settings/index.js";
import { documents } from "#graphql-documents";
import { serverOptions } from "#graphql-middleware-server-options-build";
import { useRuntimeConfig } from "#imports";
function parseMultipart(data) {
  const files = data.filter((v) => !!v.filename);
  const variablesData = data.find((v) => v.name === "variables")?.data?.toString();
  if (!variablesData) {
    throw new Error("Missing variables in form data.");
  }
  const variables = JSON.parse(variablesData);
  const map = data.find((v) => v.name === "map")?.data.toString() || "{}";
  return { files, map, variables };
}
export default defineEventHandler(async (event) => {
  const method = event.method;
  const operation = GraphqlMiddlewareOperation.Mutation;
  const operationName = event.context?.params?.name;
  validateRequest(method, operation, operationName, documents);
  const operationDocument = documents[operation][operationName];
  const multiPartData = await readMultipartFormData(event);
  if (!multiPartData) {
    return throwError("Failed to read multi part data.");
  }
  const { variables, map, files } = parseMultipart(multiPartData);
  const formData = new FormData();
  formData.append(
    "operations",
    JSON.stringify({
      query: operationDocument,
      variables,
      operationName
    })
  );
  formData.append("map", map);
  for (const file of files) {
    if (!file.name) {
      return throwError("Missing name for file: " + file.filename);
    }
    const blob = new Blob([file.data]);
    formData.append(file.name, blob, file.filename);
  }
  if (serverOptions.doGraphqlRequest) {
    return serverOptions.doGraphqlRequest({
      event,
      operation,
      operationName,
      operationDocument,
      variables,
      formData
    });
  }
  const runtimeConfig = useRuntimeConfig().graphqlMiddleware;
  const endpoint = await getEndpoint(
    runtimeConfig,
    serverOptions,
    event,
    operation,
    operationName
  );
  const fetchOptions = await getFetchOptions(
    serverOptions,
    event,
    operation,
    operationName
  );
  return $fetch.raw(endpoint, {
    ...fetchOptions,
    // @todo: Remove any once https://github.com/unjs/nitro/pull/883 is released.
    method: "POST",
    body: formData
  }).then((response) => {
    return onServerResponse(
      serverOptions,
      event,
      response,
      operation,
      operationName
    );
  }).catch((error) => {
    return onServerError(
      serverOptions,
      event,
      error,
      operation,
      operationName
    );
  });
});
